# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Persistent identifier fetchers.

A proper fetcher is defined as a function that return a
:data:`invenio_pidstore.fetchers.FetchedPID` instance.

E.g.

.. code-block:: python

    def my_fetcher(record_uuid, data):
        return FetchedPID(
            provider=MyRecordIdProvider,
            pid_type=MyRecordIdProvider.pid_type,
            pid_value=extract_pid_value(data),
        )

To see more about providers see :mod:`invenio_pidstore.providers`.
"""

from __future__ import absolute_import, print_function

from invenio_pidstore.fetchers import FetchedPID

from .providers import NREventsIdProvider


def nr_events_id_fetcher(record_uuid, data):
    """Fetch a record's identifiers.

    :param record_uuid: The record UUID.
    :param data: The record metadata.
    :returns: A :data:`invenio_pidstore.fetchers.FetchedPID` instance.
    """
    id_field = "control_number"
    return FetchedPID( # FetchedPID je obyčejný namedtuple
        provider=NREventsIdProvider,
        pid_type=NREventsIdProvider.pid_type,
        pid_value=str(data[id_field]),
    )



from invenio_indexer.api import RecordIndexer
from invenio_search import RecordsSearch


class ThesisSearch(RecordsSearch):
    pass


class EventsAPI:

    def __init__(self, app):
        """
        API initialization.

        :param app: invenio application
        """
        self.app = app
        self.indexer = RecordIndexer()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CERN.
#
# My site is free software; you can redistribute it and/or modify it under
# the terms of the MIT License; see LICENSE file for more details.

"""JSON Schemas."""

from nr_common.marshmallow.json import CommonMetadataSchemaV2
from marshmallow.fields import List, Nested

from nr_events.marshmallow.subschemas import Events


class EventsMetadataSchemaV1(CommonMetadataSchemaV2):
    events = List(Nested(Events()))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CIS UCT Prague.
#
# CIS theses repository is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Default configuration."""

from __future__ import absolute_import, print_function

from invenio_records_rest.utils import allow_all

from nr_events.record import draft_index_name

RECORDS_DRAFT_ENDPOINTS = {
    'events': {
        'draft': 'draft-events',

        'pid_type': 'nrevt',
        'pid_minter': 'nr_events',
        'pid_fetcher': 'nr_events',
        'default_endpoint_prefix': True,
        'max_result_window': 500000,
        'record_class': 'nr_events.record:PublishedEventRecord',
        'list_route': '/events/',
        'publish_permission_factory_imp': allow_all,  # TODO: change this !!!
        'unpublish_permission_factory_imp': allow_all,
        'edit_permission_factory_imp': allow_all,
        'default_media_type': 'application/json',
        'links_factory_imp': 'oarepo_fsm.links:record_fsm_links_factory'
        # 'indexer_class': CommitingRecordIndexer,

    },
    'draft-events': {
        'pid_type': 'dnrevt',
        'record_class': 'nr_events.record:DraftEventRecord',
        'list_route': '/draft/events/',
        'search_index': draft_index_name,
        'links_factory_imp': 'oarepo_fsm.links:record_fsm_links_factory'
    }
}

FILTERS = {
}

POST_FILTERS = {
}

RECORDS_REST_FACETS = {
}

RECORDS_REST_SORT_OPTIONS = {
}

RECORDS_REST_DEFAULT_SORT = {
}

"""Set default sorting options."""



import os

from flask import url_for
from invenio_records.api import Record
from nr_common.record import CommonBaseRecord
from oarepo_communities.record import CommunityRecordMixin
from oarepo_fsm.mixins import FSMMixin
from oarepo_records_draft.record import InvalidRecordAllowedMixin, DraftRecordMixin
from oarepo_references.mixins import ReferenceEnabledRecordMixin
from oarepo_validate import SchemaKeepingRecordMixin, MarshmallowValidatedRecordMixin

from .constants import EVENTS_ALLOWED_SCHEMAS, EVENTS_PREFERRED_SCHEMA
from .marshmallow import EventsMetadataSchemaV1

published_index_name = 'nr_events-nr-events-v1.0.0'
draft_index_name = 'draft-nr_events-nr-events-v1.0.0'
all_index_name = 'nr-all'

prefixed_published_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX',
                                               '') + published_index_name
prefixed_draft_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + draft_index_name
prefixed_all_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + all_index_name


class EventBaseRecord(SchemaKeepingRecordMixin,
                      MarshmallowValidatedRecordMixin,
                      ReferenceEnabledRecordMixin,
                      CommunityRecordMixin,
                      Record,
                      ):
    ALLOWED_SCHEMAS = EVENTS_ALLOWED_SCHEMAS
    PREFERRED_SCHEMA = EVENTS_PREFERRED_SCHEMA
    MARSHMALLOW_SCHEMA = EventsMetadataSchemaV1


class PublishedEventRecord(InvalidRecordAllowedMixin, EventBaseRecord):
    index_name = published_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.events_item',
                       pid_value=self['control_number'], _external=True)


class DraftEventRecord(DraftRecordMixin, EventBaseRecord):
    index_name = draft_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.draft-events_item',
                       pid_value=self['control_number'], _external=True)



from invenio_records_rest.schemas.fields import SanitizedUnicode
from nr_common.marshmallow.fields import DateRange
from nr_common.marshmallow.subschemas import TitledMixin, PublicationPlaceSchema
from invenio_records_rest.schemas import StrictKeysMixin
from marshmallow.fields import List, Nested
from oarepo_multilingual.marshmallow import MultilingualStringV2
from oarepo_taxonomies.marshmallow import TaxonomyField


class Events(StrictKeysMixin):
    nameOriginal = SanitizedUnicode(required=True)
    nameAlternate = List(SanitizedUnicode())
    nameUnified = TaxonomyField(mixins=[TitledMixin])
    date = DateRange(required=True)
    location = Nested(PublicationPlaceSchema())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nr_events',
 'nr_events.jsonschemas',
 'nr_events.jsonschemas.nr_events',
 'nr_events.mapping_includes',
 'nr_events.mapping_includes.v7',
 'nr_events.mappings',
 'nr_events.mappings.v7',
 'nr_events.marshmallow']

package_data = \
{'': ['*'], 'nr_events.mappings.v7': ['nr_events/*']}

install_requires = \
['techlib-nr-common>=3.0.0a39,<4.0.0']

entry_points = \
{'invenio_base.api_apps': ['events = nr_events:NREvents'],
 'invenio_base.apps': ['events = nr_events:NREvents'],
 'invenio_jsonschemas.schemas': ['nr_events = nr_events.jsonschemas'],
 'invenio_pidstore.fetchers': ['nr_events = '
                               'nr_events.fetchers:nr_events_id_fetcher'],
 'invenio_pidstore.minters': ['nr_events = '
                              'nr_events.minters:nr_events_id_minter'],
 'invenio_search.mappings': ['nr_events = nr_events.mappings'],
 'oarepo_mapping_includes': ['nr_events = nr_events.mapping_includes']}

setup_kwargs = {
    'name': 'techlib-nr-events',
    'version': '1.0.0a13',
    'description': 'National repository events metadata model',
    'long_description': '# nr-events\nData model for events related records\n',
    'author': 'Daniel Kopecký',
    'author_email': 'Daniel.Kopecky@techlib.cz',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# from invenio_access import authenticated_user
# from flask_principal import Permission
#
#
# def thesis_write_permission_factory(record=None, **kwargs):
#     return Permission(authenticated_user)

# TODO: dodělat až se budou řešit práva